@@ -20,6 +20,8 @@
 
 #include "config.h"
 
+#include "backends/native/meta-kms-connector.h"
+#include "backends/native/meta-kms-crtc.h"
 #include "backends/native/meta-kms-device-private.h"
 #include "backends/native/meta-kms-device.h"
 
@@ -28,7 +30,8 @@
 #include <sys/types.h>
 #include <xf86drm.h>
 
-#include "backends/native/meta-backend-native.h"
+#include "backends/native/meta-backend-native-private.h"
+#include "backends/native/meta-device-pool.h"
 #include "backends/native/meta-kms-impl-device-atomic.h"
 #include "backends/native/meta-kms-impl-device-dummy.h"
 #include "backends/native/meta-kms-impl-device-simple.h"
@@ -73,12 +76,6 @@ meta_kms_device_get_impl_device (MetaKmsDevice *device)
   return device->impl_device;
 }
 
-int
-meta_kms_device_leak_fd (MetaKmsDevice *device)
-{
-  return meta_kms_impl_device_leak_fd (device->impl_device);
-}
-
 const char *
 meta_kms_device_get_path (MetaKmsDevice *device)
 {
@@ -120,12 +117,66 @@ meta_kms_device_get_cursor_size (MetaKmsDevice *device,
     }
 }
 
+gboolean
+meta_kms_device_prefers_shadow_buffer (MetaKmsDevice *device)
+{
+  return device->caps.prefers_shadow_buffer;
+}
+
+gboolean
+meta_kms_device_uses_monotonic_clock (MetaKmsDevice *device)
+{
+  return device->caps.uses_monotonic_clock;
+}
+
 GList *
 meta_kms_device_get_connectors (MetaKmsDevice *device)
 {
   return device->connectors;
 }
 
+MetaKmsCrtc *
+meta_kms_device_find_crtc_in_impl (MetaKmsDevice *device,
+                                   uint32_t       crtc_id)
+{
+  MetaKmsImplDevice *impl_device = meta_kms_device_get_impl_device (device);
+  GList *l;
+
+  meta_assert_in_kms_impl (device->kms);
+  meta_assert_is_waiting_for_kms_impl_task (device->kms);
+
+  for (l = meta_kms_impl_device_peek_crtcs (impl_device); l; l = l->next)
+    {
+      MetaKmsCrtc *crtc = META_KMS_CRTC (l->data);
+
+      if (meta_kms_crtc_get_id (crtc) == crtc_id)
+        return crtc;
+    }
+
+  return NULL;
+}
+
+MetaKmsConnector *
+meta_kms_device_find_connector_in_impl (MetaKmsDevice *device,
+                                        uint32_t       connector_id)
+{
+  MetaKmsImplDevice *impl_device = meta_kms_device_get_impl_device (device);
+  GList *l;
+
+  meta_assert_in_kms_impl (device->kms);
+  meta_assert_is_waiting_for_kms_impl_task (device->kms);
+
+  for (l = meta_kms_impl_device_peek_connectors (impl_device); l; l = l->next)
+    {
+      MetaKmsConnector *connector = META_KMS_CONNECTOR (l->data);
+
+      if (meta_kms_connector_get_id (connector) == connector_id)
+        return connector;
+    }
+
+  return NULL;
+}
+
 GList *
 meta_kms_device_get_crtcs (MetaKmsDevice *device)
 {
@@ -179,15 +230,22 @@ meta_kms_device_get_fallback_modes (MetaKmsDevice *device)
   return device->fallback_modes;
 }
 
-void
-meta_kms_device_update_states_in_impl (MetaKmsDevice *device)
+MetaKmsUpdateChanges
+meta_kms_device_update_states_in_impl (MetaKmsDevice *device,
+                                       uint32_t       crtc_id,
+                                       uint32_t       connector_id)
 {
   MetaKmsImplDevice *impl_device = meta_kms_device_get_impl_device (device);
+  MetaKmsUpdateChanges changes;
 
   meta_assert_in_kms_impl (device->kms);
   meta_assert_is_waiting_for_kms_impl_task (device->kms);
 
-  meta_kms_impl_device_update_states (impl_device);
+  changes = meta_kms_impl_device_update_states (impl_device, crtc_id,
+                                                connector_id);
+
+  if (changes == META_KMS_UPDATE_CHANGE_NONE)
+    return changes;
 
   g_list_free (device->crtcs);
   device->crtcs = meta_kms_impl_device_copy_crtcs (impl_device);
@@ -197,6 +255,8 @@ meta_kms_device_update_states_in_impl (MetaKmsDevice *device)
 
   g_list_free (device->planes);
   device->planes = meta_kms_impl_device_copy_planes (impl_device);
+
+  return changes;
 }
 
 void
@@ -229,7 +289,6 @@ meta_kms_device_add_fake_plane_in_impl (MetaKmsDevice    *device,
 typedef struct _CreateImplDeviceData
 {
   MetaKmsDevice *device;
-  int fd;
   const char *path;
   MetaKmsDeviceFlag flags;
 
@@ -244,180 +303,73 @@ typedef struct _CreateImplDeviceData
   char *out_path;
 } CreateImplDeviceData;
 
-static gboolean
-is_atomic_allowed (const char *driver_name)
+static const char *
+impl_device_type_to_string (GType type)
 {
-  const char *atomic_driver_deny_list[] = {
-    "qxl",
-    "vmwgfx",
-    "vboxvideo",
-    "nvidia-drm",
-    "virtio_gpu",
-    NULL,
-  };
-
-  return !g_strv_contains (atomic_driver_deny_list, driver_name);
-}
-
-static gboolean
-get_driver_info (int    fd,
-                 char **name,
-                 char **description)
-{
-  drmVersion *drm_version;
-
-  drm_version = drmGetVersion (fd);
-  if (!drm_version)
-    return FALSE;
-
-  *name = g_strndup (drm_version->name,
-                     drm_version->name_len);
-  *description = g_strndup (drm_version->desc,
-                            drm_version->desc_len);
-  drmFreeVersion (drm_version);
-
-  return TRUE;
+  if (type == META_TYPE_KMS_IMPL_DEVICE_ATOMIC)
+    return "atomic modesetting";
+  else if (type == META_TYPE_KMS_IMPL_DEVICE_SIMPLE)
+    return "legacy modesetting";
+  else if (type == META_TYPE_KMS_IMPL_DEVICE_DUMMY)
+    return "no modesetting";
+  g_assert_not_reached();
 }
 
 static MetaKmsImplDevice *
 meta_create_kms_impl_device (MetaKmsDevice      *device,
                              MetaKmsImpl        *impl,
-                             int                 fd,
                              const char         *path,
                              MetaKmsDeviceFlag   flags,
                              GError            **error)
 {
-  GType impl_device_type;
-  gboolean supports_atomic_mode_setting;
-  g_autofree char *driver_name = NULL;
-  g_autofree char *driver_description = NULL;
-  const char *atomic_kms_enable_env;
-  int impl_fd;
-  g_autofree char *impl_path = NULL;
-  MetaKmsImplDevice *impl_device;
-
   meta_assert_in_kms_impl (meta_kms_impl_get_kms (impl));
 
-  if (!get_driver_info (fd, &driver_name, &driver_description))
-    {
-      driver_name = g_strdup ("unknown");
-      driver_description = g_strdup ("Unknown");
-    }
-
-  atomic_kms_enable_env = getenv ("MUTTER_DEBUG_ENABLE_ATOMIC_KMS");
-
   if (flags & META_KMS_DEVICE_FLAG_NO_MODE_SETTING)
     {
-      g_autofree char *render_node_path = NULL;
-
-      render_node_path = drmGetRenderDeviceNameFromFd (fd);
-      if (!render_node_path)
-        {
-          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                       "Couldn't find render node device for '%s' (%s)",
-                       path, driver_name);
-          return NULL;
-        }
-
-      impl_fd = open (render_node_path, O_RDWR | O_CLOEXEC, 0);
-      if (impl_fd == -1)
-        {
-          g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errno),
-                       "Failed to open render node '%s': %s",
-                       render_node_path, g_strerror (errno));
-          return NULL;
-        }
-
-      g_message ("Adding device '%s' (from '%s', %s) using no mode setting.",
-                 render_node_path, path, driver_name);
-
-      impl_path = g_steal_pointer (&render_node_path);
-      impl_device_type = META_TYPE_KMS_IMPL_DEVICE_DUMMY;
-    }
-  else if (atomic_kms_enable_env)
-    {
-      if (g_strcmp0 (atomic_kms_enable_env, "1") == 0)
-        {
-          impl_device_type = META_TYPE_KMS_IMPL_DEVICE_ATOMIC;
-          if (drmSetClientCap (fd, DRM_CLIENT_CAP_ATOMIC, 1) != 0)
-            {
-              g_error ("Failed to force atomic mode setting on '%s' (%s).",
-                       path, driver_name);
-            }
-        }
-      else if (g_strcmp0 (atomic_kms_enable_env, "0") == 0)
-        {
-          impl_device_type = META_TYPE_KMS_IMPL_DEVICE_SIMPLE;
-        }
-      else
-        {
-          g_error ("Invalid value '%s' for MUTTER_DEBUG_ENABLE_ATOMIC_KMS, "
-                   "bailing.",
-                   atomic_kms_enable_env);
-        }
-
-      impl_fd = dup (fd);
-      impl_path = g_strdup (path);
-
-      g_message ("Mode setting implementation for '%s' (%s) forced (%s).",
-                 path, driver_name,
-                 impl_device_type == META_TYPE_KMS_IMPL_DEVICE_ATOMIC ?
-                   "atomic" : "non-atomic");
-    }
-  else if (!is_atomic_allowed (driver_name))
-    {
-      g_message ("Adding device '%s' (%s) using non-atomic mode setting"
-                 " (using atomic mode setting not allowed).",
-                 path, driver_name);
-      impl_fd = dup (fd);
-      impl_path = g_strdup (path);
-      impl_device_type = META_TYPE_KMS_IMPL_DEVICE_SIMPLE;
+      return g_initable_new (META_TYPE_KMS_IMPL_DEVICE_DUMMY,
+                             NULL, error,
+                             "device", device,
+                             "impl", impl,
+                             "path", path,
+                             "flags", flags,
+                             NULL);
     }
   else
     {
-      int ret;
+      GType impl_device_types[] = {
+        META_TYPE_KMS_IMPL_DEVICE_ATOMIC,
+        META_TYPE_KMS_IMPL_DEVICE_SIMPLE,
+      };
+      int i;
 
-      ret = drmSetClientCap (fd, DRM_CLIENT_CAP_ATOMIC, 1);
-      if (ret == 0)
-        supports_atomic_mode_setting = TRUE;
-      else
-        supports_atomic_mode_setting = FALSE;
+      for (i = 0; i < G_N_ELEMENTS (impl_device_types); i++)
+        {
+          MetaKmsImplDevice *impl_device;
+          g_autoptr (GError) local_error = NULL;
 
-      if (supports_atomic_mode_setting)
-        {
-          g_message ("Adding device '%s' (%s) using atomic mode setting.",
-                     path, driver_name);
-          impl_device_type = META_TYPE_KMS_IMPL_DEVICE_ATOMIC;
-        }
-      else
-        {
-          g_message ("Adding device '%s' (%s) using non-atomic mode setting.",
-                     path, driver_name);
-          impl_device_type = META_TYPE_KMS_IMPL_DEVICE_SIMPLE;
+          impl_device = g_initable_new (impl_device_types[i],
+                                        NULL, &local_error,
+                                        "device", device,
+                                        "impl", impl,
+                                        "path", path,
+                                        "flags", flags,
+                                        NULL);
+          if (impl_device)
+            return impl_device;
+
+          if (local_error->domain != META_KMS_ERROR)
+            {
+              g_warning ("Failed to open %s backend: %s",
+                         impl_device_type_to_string (impl_device_types[i]),
+                         local_error->message);
+            }
         }
 
-      impl_fd = dup (fd);
-      impl_path = g_strdup (path);
-    }
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "No suitable mode setting backend found");
 
-  impl_device = g_initable_new (impl_device_type, NULL, error,
-                                "device", device,
-                                "impl", impl,
-                                "fd", impl_fd,
-                                "path", impl_path,
-                                "flags", flags,
-                                "driver-name", driver_name,
-                                "driver-description", driver_description,
-                                NULL);
-  if (!impl_device)
-    {
-      close (impl_fd);
       return NULL;
     }
-
-  close (fd);
-
-  return impl_device;
 }
 
 static gpointer
@@ -430,7 +382,6 @@ create_impl_device_in_impl (MetaKmsImpl  *impl,
 
   impl_device = meta_create_kms_impl_device (data->device,
                                              impl,
-                                             data->fd,
                                              data->path,
                                              data->flags,
                                              error);
@@ -461,46 +412,20 @@ meta_kms_device_new (MetaKms            *kms,
                      MetaKmsDeviceFlag   flags,
                      GError            **error)
 {
-  MetaBackend *backend = meta_kms_get_backend (kms);
-  MetaBackendNative *backend_native = META_BACKEND_NATIVE (backend);
-  MetaLauncher *launcher = meta_backend_native_get_launcher (backend_native);
   MetaKmsDevice *device;
   CreateImplDeviceData data;
-  int fd;
-
-  if (flags & META_KMS_DEVICE_FLAG_NO_MODE_SETTING)
-    {
-      fd = open (path, O_RDWR | O_CLOEXEC, 0);
-      if (fd == -1)
-        {
-          g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errno),
-                       "Failed to open DRM device: %s", g_strerror (errno));
-          return NULL;
-        }
-    }
-  else
-    {
-      fd = meta_launcher_open_restricted (launcher, path, error);
-      if (fd == -1)
-        return NULL;
-    }
 
   device = g_object_new (META_TYPE_KMS_DEVICE, NULL);
   device->kms = kms;
 
   data = (CreateImplDeviceData) {
     .device = device,
-    .fd = fd,
     .path = path,
     .flags = flags,
   };
   if (!meta_kms_run_impl_task_sync (kms, create_impl_device_in_impl, &data,
                                     error))
     {
-      if (flags & META_KMS_DEVICE_FLAG_NO_MODE_SETTING)
-        close (fd);
-      else
-        meta_launcher_close_restricted (launcher, fd);
       g_object_unref (device);
       return NULL;
     }
@@ -521,27 +446,15 @@ meta_kms_device_new (MetaKms            *kms,
   return device;
 }
 
-typedef struct _FreeImplDeviceData
-{
-  MetaKmsImplDevice *impl_device;
-
-  int out_fd;
-} FreeImplDeviceData;
-
 static gpointer
 free_impl_device_in_impl (MetaKmsImpl  *impl,
                           gpointer      user_data,
                           GError      **error)
 {
-  FreeImplDeviceData *data = user_data;
-  MetaKmsImplDevice *impl_device = data->impl_device;
-  int fd;
+  MetaKmsImplDevice *impl_device = user_data;
 
-  fd = meta_kms_impl_device_close (impl_device);
   g_object_unref (impl_device);
 
-  data->out_fd = fd;
-
   return GINT_TO_POINTER (TRUE);
 }
 
@@ -549,9 +462,6 @@ static void
 meta_kms_device_finalize (GObject *object)
 {
   MetaKmsDevice *device = META_KMS_DEVICE (object);
-  MetaBackend *backend = meta_kms_get_backend (device->kms);
-  MetaBackendNative *backend_native = META_BACKEND_NATIVE (backend);
-  MetaLauncher *launcher = meta_backend_native_get_launcher (backend_native);
 
   g_free (device->path);
   g_list_free (device->crtcs);
@@ -560,26 +470,11 @@ meta_kms_device_finalize (GObject *object)
 
   if (device->impl_device)
     {
-      FreeImplDeviceData data;
-      GError *error = NULL;
-
-      data = (FreeImplDeviceData) {
-        .impl_device = device->impl_device,
-      };
-      if (!meta_kms_run_impl_task_sync (device->kms, free_impl_device_in_impl, &data,
-                                        &error))
-        {
-          g_warning ("Failed to close KMS impl device: %s", error->message);
-          g_error_free (error);
-        }
-      else
-        {
-          if (device->flags & META_KMS_DEVICE_FLAG_NO_MODE_SETTING)
-            close (data.out_fd);
-          else
-            meta_launcher_close_restricted (launcher, data.out_fd);
-        }
+      meta_kms_run_impl_task_sync (device->kms, free_impl_device_in_impl,
+                                   device->impl_device,
+                                   NULL);
     }
+
   G_OBJECT_CLASS (meta_kms_device_parent_class)->finalize (object);
 }
 
